//------------------------------------------------------------------------------
// File: BonTuner.h
//   Define CBonTuner class
//------------------------------------------------------------------------------
#pragma once

#include <Windows.h>
#include <atlbase.h>
#include <dshow.h>
#include <tuner.h>

#pragma warning(disable : 4995)
#include <string>
#include <sstream>
using namespace std;

#include "IBonDriver2.h"

#define GUID_DVBS_TUNINGSPACE L"{B64016F3-C9A2-4066-96F0-BD9563314726}"

void OutputDebug(LPCSTR format,...);
class IBdaSpecials;
interface IGrabTsForBD;
// CBonTuner class
////////////////////////////////
class CBonTuner : public IBonDriver2
{
public:
	CBonTuner();
	virtual ~CBonTuner();

	// IBonDriver
	const BOOL OpenTuner(void);
	void CloseTuner(void);

	const BOOL SetChannel(const BYTE byCh);
	const float GetSignalLevel(void);

	const DWORD WaitTsStream(const DWORD dwTimeOut = 0);
	const DWORD GetReadyCount(void);

	const BOOL GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain);
	const BOOL GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain);

	void PurgeTsStream(void);

	// IBonDriver2
	LPCTSTR GetTunerName(void);

	const BOOL IsTunerOpening(void);

	LPCTSTR EnumTuningSpace(const DWORD dwSpace);
	LPCTSTR EnumChannelName(const DWORD dwSpace, const DWORD dwChannel);

	const BOOL SetChannel(const DWORD dwSpace, const DWORD dwChannel);

	const DWORD GetCurSpace(void);
	const DWORD GetCurChannel(void);

	void Release(void);
	
	static CBonTuner *m_pThis;
	static HINSTANCE m_hModule;

protected: // Internal subroutines

	void ReadIniFile(void);
	// INI t@CŎw肷`[ip[^
	struct TunerParam {
		wstring sTunerGUID;
		wstring sTunerName;
		wstring sDLLBaseName;
	} m_sTunerParam;

	//
	// Graph ֘A
	//
	ITuningSpace *m_pITuningSpace;
	IBaseFilter *m_pNetworkProvider;
	IBaseFilter *m_pTunerDevice;
	IBaseFilter *m_pCaptureDevice;
	IBaseFilter *m_pGrabber;
	IGrabTsForBD *m_pIGrabTsForBD;
	IBaseFilter *m_pDemux;
	IBaseFilter *m_pTif;
	IGraphBuilder *m_pFilterGraph;
	IMediaControl *m_pIMediaControl;

	HRESULT InitializeGraphBuilder(void);
	HRESULT RunGraph(void);
	void StopGraph(void);
	void CleanupGraph(void);

	// TuningSpace
	HRESULT LoadTuningSpace(void);
	void UnloadTuningSpace(void);
	HRESULT CreateTuningSpace(void);
	HRESULT InitTuningSpace(void);

	// NetworkProvider
	HRESULT LoadNetworkProvider(void);
	void UnloadNetworkProvider(void);

	// TunerDevice
	HRESULT LoadAndConnectTunerDevice(void);
	void UnloadTunerDevice(void);
	
	// CaptureDevice
	HRESULT LoadAndConnectCaptureDevice(void);
	void UnloadCaptureDevice(void);
	
	// Grabber
	HRESULT LoadAndConnectGrabDevice(void);
	void UnloadGrabDevice(void);

	// Demultiplexer
	HRESULT LoadAndConnectDemux(void);
	void UnloadDemux(void);
	
	// TIF (Transport Information Filter)
	HRESULT LoadAndConnectTif(void);
	void UnloadTif(void);

	HRESULT Connect(const char* pszName, IBaseFilter* pFrom,
		IBaseFilter* pTo);

	BOOL LockChannel(BYTE bySatellite, BOOL bHorizontal, unsigned long ulFrequency, BOOL bDvbS2);
	void GetSignalState(int* pnStrength, int* pnQuality, int* pnLock);

	static DWORD WINAPI TimerThread(LPVOID pParam);

	BOOL m_bOpened; // Tuner is opened
	HANDLE m_hSemaphore; // rp

	HANDLE m_hTimerThread;
	BOOL m_bLoopIoThread;
	HANDLE m_hOnStreamEvent;
	static const DWORD POLLINGWAIT = 100UL; // 񓯊NGXg|[OԊu(ms)

	// channel table
	static const int MAX_SERVICE = 512;
	struct {
		WCHAR szServiceName[64];
		BYTE bySatellite; // 0 .. JCSAT-3, 1 .. JCSAT-4
		BOOL bHorizontal;
		BOOL bDvbS2;
		DWORD dwFrequency;
	} m_aCh[MAX_SERVICE];

	static const int MAX_TP = 40;
	static const DWORD CHANNEL_INVALID = 0xFFFFFFFF;

	DWORD m_dwNumChannel;
	DWORD m_dwCurChannel; // current channel
	HMODULE m_hModuleTunerSpecials; // TunerSpecial DLL module handle
	IBdaSpecials *m_pIBdaSpecials;

	void CheckAndSetTunerDependCode(wstring &displayName);

#ifdef _DEBUG
	void printFiltersRefferenceCount(const int lineNum);
	static int FLAG;
#endif
};
