//------------------------------------------------------------------------------
// File: DSFilterEnum.cpp
//   Implementation of CDSFilterEnum class
//------------------------------------------------------------------------------

#include <Windows.h>
#include <atlbase.h>
#include <dshow.h>
#include <ks.h>
#include <ksmedia.h>
#include <bdatypes.h>
#include <bdamedia.h>

#pragma warning(disable : 4995)
#include <string>
#include <sstream>
using namespace std;

#include "DSFilterEnum.h"

CDSFilterEnum::CDSFilterEnum(CLSID clsid, bool bDvbHack)
: m_pIEnumMoniker(NULL), m_pICreateDevEnum(NULL),
  m_pIMoniker(NULL), bDvbHack(bDvbHack)
{
	HRESULT hr = ::CoCreateInstance(CLSID_SystemDeviceEnum,
		NULL,
		CLSCTX_INPROC,
		IID_ICreateDevEnum,
		reinterpret_cast<void**>(&m_pICreateDevEnum));

	if (FAILED(hr)) {
		string e("Error in CoCreateInstance.");
		throw e;
		return; /* not reached */
	}

    hr = m_pICreateDevEnum->CreateClassEnumerator(clsid, &m_pIEnumMoniker, 0);

	if ((FAILED (hr)) || (hr != S_OK)) {
		// CreateClassEnumerator Ȃ || Ȃ
		m_pICreateDevEnum->Release();
		m_pICreateDevEnum = NULL;
		string e("Error in CoCreateInstance.");
		throw e;
		return; /* not reached */
	}

	return;
}

CDSFilterEnum::~CDSFilterEnum(void)
{
	if (m_pIMoniker) {
		m_pIMoniker->Release();
		m_pIMoniker = NULL;
	}

	if (m_pICreateDevEnum) {
		m_pICreateDevEnum->Release();
	}

	if (m_pIEnumMoniker) {
		m_pIEnumMoniker->Release();
	}
}

HRESULT CDSFilterEnum::next(void)
{
	if (m_pIMoniker) {
		m_pIMoniker->Release();
		m_pIMoniker = NULL;
	}

	HRESULT hr;
	while ((hr = m_pIEnumMoniker->Next(1, &m_pIMoniker, 0)) == S_OK) {
#define DVBWORLD_HACK
		// 2chX4:82-(2) ̑΍, eXg
#ifdef DVBWORLD_HACK
		if (!bDvbHack)
			break;
		wstring displayName;
		this->getDisplayName(&displayName);
		if (displayName.find(L"@device:pnp:") != string::npos) {
			OutputDebugStringA("BDA Tuner Device found.\n");
			break;
		} else {
			OutputDebugStringA("maybe WDM tuner device of DVBWorld?\n");
			// try "Next"
		}
#else
		break;
#endif

	}

	return hr;
}


HRESULT CDSFilterEnum::getFilter(IBaseFilter** ppFilter)
{
	if ((!ppFilter) || (m_pIMoniker == NULL))
		return E_POINTER;

	*ppFilter = NULL;

	// bind the filter        
    HRESULT hr = m_pIMoniker->BindToObject(
		NULL, 
		NULL, 
		IID_IBaseFilter,
		reinterpret_cast<void**>(ppFilter));

	if (!*ppFilter)
			return E_POINTER;

	return hr; 
}

HRESULT CDSFilterEnum::getFriendlyName(wstring* pName)
{
	if (m_pIMoniker == NULL) {
		return E_POINTER;
	}

    IPropertyBag *pBag;
    HRESULT hr =m_pIMoniker->BindToStorage(
		NULL, 
		NULL, 
		IID_IPropertyBag,
		reinterpret_cast<void**>(&pBag));

	if(FAILED(hr)) {
		OutputDebugStringA("Cannot BindToStorage for.\n");
        return hr;
    }

   	VARIANT varBSTR;
	varBSTR.vt = VT_BSTR;		
	VariantInit(&varBSTR);			

	hr = pBag->Read(L"FriendlyName", &varBSTR, NULL);	

	if(FAILED(hr)){
		OutputDebugStringA("IPropertyBag->Read method failed for.\n");
		pBag->Release();		
		VariantClear(&varBSTR);
		return hr;
    }
	
	*pName = varBSTR.bstrVal;
	
	pBag->Release();		
	VariantClear(&varBSTR);

	return S_OK;
}

HRESULT CDSFilterEnum::getDisplayName(wstring* pName)
{
	if (m_pIMoniker == NULL) {
		return E_POINTER;
	}

	LPOLESTR pwszName;
	if ((m_pIMoniker->GetDisplayName(NULL, NULL, &pwszName)) != S_OK) {
		return E_FAIL;
	}
	CComBSTR bstrName = pwszName;
	bstrName.ToLower();
	*pName = bstrName.m_str;

	return S_OK;
}
